from django.contrib import admin, auth
from django.contrib.auth.admin import UserAdmin

from .models import User

class MunityUserAdmin(UserAdmin):
    def get_form(self, request, obj=None, change=False, **kwargs):
        kwargs['labels'] = {
            'groups': 'Roles'
        }
        return super().get_form(request, obj=obj, change=change, **kwargs)

admin.site.register(User)

admin.site.unregister(auth.models.Group)


import uuid

from django.contrib.auth.models import AbstractUser
from django.db import models
from django.db.models.deletion import SET_NULL

from munity.authorization.models import Role
from ..models import MunityGroupableModel, MunityModel

class User(AbstractUser, MunityModel, MunityGroupableModel):
    roles = models.ManyToManyField("authorization.Role", through="UserRoleWorkspace", related_name="roles")
    def get_absolute_url(self):
        return f"/users/{self.id}"

class UserRoleWorkspace(models.Model):
    user = models.ForeignKey("users.User", on_delete=models.CASCADE)
    workspace = models.ForeignKey("workspaces.Workspace", on_delete=models.CASCADE, null=True)
    role = models.ForeignKey("authorization.Role", on_delete=models.CASCADE)

    class Meta:
        unique_together=('user', 'workspace', 'role')

class UserRoleGenericGroup(models.Model):
    user = models.ForeignKey("users.User", on_delete=models.CASCADE)
    group = models.ForeignKey("generic_groups.GenericGroup", on_delete=models.CASCADE)
    role = models.ForeignKey("authorization.Role", on_delete=models.CASCADE)

    class Meta:
        unique_together=('user', 'group', 'role')



from django_filters import rest_framework as filters
from munity.views import MunityGroupableMixin, MunityViewSet
from rest_framework import serializers, viewsets
from django.db.models import Q

from .models import User


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        workspace_id = serializers.SlugField(write_only=True)
        # fields = '__all__'
        fields = [
            "id",
            "workspace",
            "username",
            "email",
            "roles",
            "first_name",
            "last_name",
            "created",
            "is_superuser",
            "modified",
            "generic_groups",
        ]
        model = User
        depth = 1

class UsersFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "workspace": ["exact", "in"],
            "first_name": ["exact", "in", "contains"],
            "generic_groups": ["in"],
            "last_name": ["exact", "in", "contains"],
            "email": ["exact", "in", "contains"],
            "username": ["exact", "in", "contains"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = User


class UsersViewSet(MunityViewSet, MunityGroupableMixin):
    serializer_class = UserSerializer
    filterset_class = UsersFilter

    def perform_create(self, serializer):
        MunityGroupableMixin.perform_create(self, serializer)
        MunityViewSet.perform_create(self, serializer)
    def perform_update(self, serializer):
        MunityGroupableMixin.perform_update(self, serializer)
        MunityViewSet.perform_update(self, serializer)
    def perform_destroy(self, instance):
        MunityGroupableMixin.perform_destroy(self, instance)
        MunityViewSet.perform_destroy(self, instance)





import uuid

# from .groups.models import Group

from django.db import models
from django_extensions.db.models import TimeStampedModel


class MunityModel(TimeStampedModel):
    """
    Default base class for munity models
    use uuid for ids
    add creation and update date
    """

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    workspace = models.ForeignKey(
        "workspaces.Workspace", to_field="slug", on_delete=models.CASCADE, null=True, blank=True
    )

    class Meta:
        abstract = True

class MunityGroupableModel(models.Model):
    """
    Default base class for munity models
    use uuid for ids
    add creation and update date
    add logs
    add groups
    """
    generic_groups = models.ManyToManyField('generic_groups.GenericGroup', blank=True)

    class Meta:
        abstract = True


import re

def get_request_info(request):
    # return (workspace_slug, ressource_type, action)

    if (request.method == "GET"):
        action = "list"
    if (request.method in ["PATCH", "PUT"]):
        action = "update"
    if (request.method == "DELETE"):
        action = "delete"
    if (request.method == "POST"):
        action = "create"

    ask_for_all_ws = re.search('^\/v[0-9]+\/workspaces/$', request.path)
    if (ask_for_all_ws) :
        return (None, "workspaces", action)

    ask_for_specific_ws = re.search('^\/v[0-9]+\/workspaces/([^\/]+)/$', request.path)
    if (ask_for_specific_ws) :
        return (ask_for_specific_ws.group(1), "workspaces", action)

    ask_for_specific_ws_ressource = re.search('^\/v[0-9]+\/workspaces/([^\/]+)\/([a-z\-]+)\/', request.path)
    if (ask_for_specific_ws_ressource) :
        return (ask_for_specific_ws_ressource.group(1), ask_for_specific_ws_ressource.group(2), action)

    ask_for_all_ressource = re.search('^\/v[0-9]+\/([^\/]+)\/', request.path)
    if (ask_for_all_ressource) :
        return (None, ask_for_all_ressource.group(1), action)



from rest_framework import viewsets
from munity.generic_groups.models import GenericGroup
from django.db.models.query_utils import Q

from munity.workspaces.models import Workspace

# Create your views here.
class MunityViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        model = self.serializer_class.Meta.model

        if "workspace_pk" in self.kwargs:
            return model.objects.filter(Q(workspace=self.kwargs["workspace_pk"]))
        return model.objects.all()

    def perform_destroy(self, instance):
        pass

    def perform_create(self, serializer):
        instance = serializer.save()
        data = self.request.data
        if "workspace" in data:
            instance.workspace_id = data["workspace"]
            instance.save()

    def perform_update(self, serializer):
        instance = serializer.save()
        data = self.request.data
        if "workspace" in data:
            instance.workspace_id = data["workspace"]
            instance.save()

class MunityGroupableMixin(viewsets.ModelViewSet):
    def perform_create(self, serializer):
        instance = serializer.save()
        data = self.request.data
        if "generic_groups" in data:
            groups = GenericGroup.objects.filter(pk__in=data["generic_groups"])
            instance.generic_groups.set(groups)

    def perform_update(self, serializer):
        instance = serializer.save()
        data = self.request.data
        if "generic_groups" in data:
            groups = GenericGroup.objects.filter(pk__in=data["generic_groups"])
            instance.generic_groups.set(groups)

    def perform_destroy(self, instance):
        instance.generic_groups.set([])
        instance.delete()